/*
 * Copyright (c) 2002 Patrick Tullmann
 * All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this file for any
 * purpose with or without restriction is hereby granted.
 */

public final class Direction
{
	public static final Direction North = new Direction("N", 0, 1);
	public static final Direction South = new Direction("S", 0, -1);
	public static final Direction West = new Direction("W", -1, 0);
	public static final Direction East = new Direction("E", 1, 0);

	final String dir;
	private final int dx;
	private final int dy;
	
	public static Direction forstr(String s)
	{
		if (s.equals("N"))
			return North;
		if (s.equals("S"))
			return South;
		if (s.equals("W"))
			return West;
		if (s.equals("E"))
			return East;

		throw new Error("Unexpected direction character: " +s);
	}

	private Direction(String s, int dx, int dy)
	{
		this.dir = s;
		this.dx = dx;
		this.dy = dy;
	}

	public Direction rotateRight()
	{
		if (this == North)
			return East;
		if (this == East)
			return South;
		if (this == South)
			return West;
		if (this == West)
			return North;
		return North; // XXX
	}

	public int dx()
	{
		return this.dx;
	}

	public int dy()
	{
		return this.dy;
	}

	public static Direction fromTo(int srcx, int srcy,
				       int destx, int desty)
	{
		if (srcx < destx)
			return Direction.East;
		if (srcx > destx)
			return Direction.West;
		if (srcy < desty)
			return Direction.North;
		if (srcy > desty)
			return Direction.South;
		
		// XXX ?
		throw new Error("No where to go.");
	}

	public String toString()
	{
		return dir;
	}
}
	
