/*
 * Copyright (c) 2002 Patrick Tullmann
 * All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this file for any
 * purpose with or without restriction is hereby granted.
 */

public final class JExpress
{
	Server server;

	public JExpress(String serverHost, int serverPort)
	{
		this.server = new Server(serverHost, serverPort);
	}
	
	private static final boolean DEBUGPRINTS = false;

	public void run()
	{
		MemUsage memUsage = new MemUsage();

		Robot robot = Robot.make("BaseHoming");
		Board board = new Board(); // XXX ask robot to create board...
		//Robot robot = Robot.make("Noop");
		WorldState worldState = new WorldState(robot);

		memUsage.record("post setup");
			
		try
		{
			this.server.establish();
			System.out.println("Established connection.");
			this.server.getBoard(board);
			this.server.getMyState(robot);
			this.server.updateWorldState(worldState);
			
			if (DEBUGPRINTS)
			{
				board.dump(System.out);
				//System.err.println("Got all the state.");
			}
			
			while (robot.funds() > 0)
			{
				//System.err.println("Turn...");
				memUsage.record("turn");
				Package[] currPkgs = this.server.getCurrentPackages();
				worldState.updatePackages(currPkgs, robot.x(), robot.y());
				this.server.sendCommand(robot.command(board, worldState, currPkgs));
				worldState.markOthers();
				this.server.updateWorldState(worldState);
				worldState.sweepOthers();
				if (DEBUGPRINTS)
				{
					worldState.printOthers(System.out);
				}
				board.decayPenalities();

				//throw new Error("Only once.");
			}
			
			System.out.println("Ran out of funds.");
			this.server.getCurrentPackages();
		}
		catch (EndOfGame eog)
		{
			System.out.println("Server has tired of your antics.");
		}
		finally
		{
			memUsage.record("end of game");
			memUsage.report();
			this.server.flush();
			this.server.close();
			//System.exit(0);
		}
	}

	public static void main(String[] args)
	{
		if (args.length != 2)
			throw new Error("Usage: java JExpress <host> <port>");
		
		new JExpress(args[0], Integer.parseInt(args[1])).run();
	}
}
